<?php
/**
 * @file
 * Contains the SunMailerNewsletter class.
 */

/**
 * Represents a newsletter.
 */
class SunMailerNewsletter {
  /**
   * Newsletter ID stored in the database.
   *
   * @var int
   */
  private $nlid;

  /**
   * Subject line stored in the database.
   *
   * @var string
   */
  private $subject;

  /**
   * Title stored in the database.
   *
   * @var string
   */
  private $title;

  /**
   * File ID stored in the database for the logo.
   *
   * @var int
   */
  private $logo_fid;

  /**
   * Date format stored in the database.
   *
   * @var string
   */
  private $date_format;

  /**
   * Message stored in the database.
   *
   * @var string
   */
  private $message;

  /**
   * Cutoff date stored in the database.
   *
   * @var int
   */
  private $cutoff_date;

  /**
   * Design ID stored in the database.
   *
   * @var int
   */
  private $did;

  /**
   * Alternate design ID stored in the database.
   *
   * @var int
   */
  private $alt_did;

  /**
   * Weight stored in the database.
   *
   * @var unknown_type
   */
  private $weight;

  /**
   * Logo corresponding to the file ID stored in the database.
   *
   * @var object.
   */
  private $logo;

  /**
   * Design corresponding to the design ID stored in the database.
   *
   * @var SunMailerDesign
   */
  private $design;

  /**
   * Alternate design corresponding to the design ID stored in the database.
   *
   * @var SunMailerDesign
   */
  private $alternateDesign;

  /**
   * Sections corresponding to this newsletter.
   *
   * @var array
   */
  private $sections;

  /**
   * Original file ID loaded from the database for the XSL.
   *
   * @var int
   */
  private $oldLogoFid;

  /**
   * Initializes a new instance of the SunMailerNewsletter class.
   *
   * @param $newsletter
   *   Object containing the newsletter's data.
   */
  private function __construct($newsletter) {
    foreach ($newsletter as $key => $value) {
      $this->$key = $newsletter->$key;
    }
    $this->logo = NULL;
    $this->design = NULL;
    $this->alternateDesign = NULL;
    $this->sections = NULL;
  }


  /**
   * Creates a new newsletter.
   *
   * @return
   *   New newsletter.
   */
  public static function createNewsletter() {
    $newsletter = new stdClass();
    $newsletter->nlid = NULL;
    $newsletter->subject = '';
    $newsletter->title = '';
    $newsletter->logo_fid = 0;
    $newsletter->date_format = 'l, F j, Y';
    $newsletter->message = '';
    $newsletter->cutoff_date = time();
    $newsletter->did = 1;
    $newsletter->alt_did = 0;
    $newsletter->weight = 0;
    return new SunMailerNewsletter($newsletter);
  }

  /**
   * Loads a newsletter from the database.
   *
   * @param $nlid
   *   Newsletter's ID.
   *
   * @return
   *   Existing newsletter or NULL if the newsletter was not found.
   */
  public static function loadNewsletter($nlid) {
    $query = "SELECT * FROM {sunmailer_newsletters} WHERE nlid = %d";
    $newsletters = SunMailerNewsletter::loadNewsletters($query, $nlid);
    return count($newsletters) ? $newsletters[$nlid] : NULL;
  }

  /**
   * Loads all newsletters from the database in weighted order.
   *
   * @return
   *   All newsletters.
   */
  public static function loadAllNewsletters() {
    $query = "SELECT * FROM {sunmailer_newsletters} ORDER BY weight, nlid ASC";
    return SunMailerNewsletter::loadNewsletters($query);
  }

  /**
   * Deletes a newsletter from the database.
   *
   * @param $nlid
   *   ID of the newsletter to delete.
   */
  public static function deleteNewsletter($nlid) {
    $newsletter = SunMailerSection::loadSection($nlid);
    if (!$newsletter) {
      return;
    }
    db_query("DELETE FROM {sunmailer_newsletters} WHERE nlid = %d", $newsletter->getNewsletterId());
    db_query("DELETE FROM {files} WHERE fid = %d", $newsletter->getLogoFileId());
    db_query("DELETE FROM {sunmailer_sections} WHERE nlid = %d", $newsletter->getNewsletterId());
  }

  /**
   * Gets the newsletter's ID.
   *
   * @return
   *   Newsletter's ID.
   */
  public function getNewsletterId() {
    return $this->nlid;
  }

  /**
   * Gets the newsletter's subject line.
   *
   * @return
   *   Newsletter's subject line.
   */
  public function getSubject() {
    return $this->subject;
  }

  /**
   * Sets the newsletters' subject line.
   *
   * @param $subject
   *   Newsletter's subject line.
   */
  public function setSubject($subject) {
    $this->subject = $subject;
  }

  /**
   * Gets the newsletter's title.
   *
   * @return
   *   Newsletter's title.
   */
  public function getTitle() {
    return $this->title;
  }

  /**
   * Sets the newsletter's title.
   *
   * @param $title
   *   Newsletter's title.
   */
  public function setTitle($title) {
    $this->title = $title;
  }

  /**
   * Checks if the newsletter has a logo.
   *
   * @return
   *   True if the newsletter has a logo.
   */
  public function hasLogo() {
    return $this->getLogo() != NULL;
  }

  /**
   *  Gets the file ID for the newsletter's logo.
   *
   *  @return
   *    File ID for the newsletter's logo.
   */
  public function getLogoFileId() {
    return $this->logo_fid;
  }

  /**
   * Gets the newsletter's logo.
   *
   * @return
   *   Newletter's logo.
   */
  public function getLogo() {
    if (!$this->logo_fid) {
      return NULL;
    }
    if (!$this->logo) {
      $query = "SELECT * FROM {files} WHERE fid = %d";
      $this->logo = db_fetch_object(db_query($query, $this->logo_fid));
    }
    return $this->logo;
  }

  /**
   * Sets the file ID for the newsletter's logo.
   *
   * @param $logoFid
   *   File ID for the newsletter's logo.
   */
  public function setLogoFileId($logoFid) {
    $this->logo = NULL;
    $this->logo_fid = $logoFid;
  }

  /**
   * Deletes the newsletter's logo.
   */
  public function deleteLogo() {
    $this->setLogoFileId(0);
  }

  /**
   * Gets the format of the newsletter's date.
   *
   * @return
   *   Format of the newsletter's date.
   */
  public function getDateFormat() {
    return $this->date_format;
  }

  /**
   * Sets the format of the newsletter's date.
   *
   * @param $dateFormat
   *   Format of the newsletter's date.
   */
  public function setDateFormat($dateFormat) {
    $this->date_format = $dateFormat;
  }

  /**
   * Checks if the newsletter has a message.
   *
   * @return
   *   True if the newsletter has a message.
   */
  public function hasMessage() {
    return $this->message != '';
  }

  /**
   * Gets the newsletter's message.
   *
   * @return
   *   Newsletter's message.
   */
  public function getMessage() {
    return $this->message;
  }

  /**
   * Sets the newsletter's message.
   *
   * @param $message
   *   Newsletter's message.
   */
  public function setMessage($message) {
    $this->message = $message;
  }

  /**
   * Deletes the newsletter's message.
   */
  public function deleteMessage() {
    $this->message = '';
  }

  /**
   * Checks if the newsletter has a cutoff date.
   *
   * @return
   *   True if the newsletter has a cutoff date.
   */
  public function hasCutoffDate() {
    return $this->cutoff_date != 0;
  }

  /**
   * Gets the newsletter's cutoff date.
   *
   * @return
   *   Newsletter's cutoff date.
   */
  public function getCutoffDate() {
    return $this->cutoff_date;
  }

  /**
   * Sets the newsletter's cutoff date.
   *
   * @param $cutoffDate
   *   Newsletter's cutoff date.
   */
  public function setCutoffDate($cutoffDate) {
    $this->cutoff_date = $cutoffDate;
  }

  /**
   * Deletes the newsletter's cutoff date.
   */
  public function deleteCutoffDate() {
    $this->cutoff_date = 0;
  }

  /**
   * Gets the ID for the newsletter's design.
   *
   * @return
   *   ID for the newsletter's design.
   */
  public function getDesignId() {
    return $this->did;
  }

  /**
   * Gets the newsletter's design.
   *
   * @return
   *   Newsletter's design.
   */
  public function getDesign() {
    if (!$this->design) {
      $this->design = SunMailerDesign::loadDesign($this->did);
    }
    return $this->design;
  }

  /**
   * Sets the ID for the newsletter's design.
   *
   * @param $did
   *   ID for the newsletter's design.
   */
  public function setDesignId($did) {
    $this->design = NULL;
    $this->did = $did;
  }

  /**
   * Checks if the newsletter has an alternate design.
   *
   * @return
   *   True if the newsletter has an alternate design.
   */
  public function hasAlternateDesign() {
    return $this->getAlternateDesign() != NULL;
  }

  /**
   * Gets the ID for the newsletter's alternate design.
   *
   * @return
   *   ID for the newsletter's alternate design.
   */
  public function getAlternateDesignId() {
    return $this->alt_did;
  }

  /**
   * Gets the newsletter's alternate design.
   *
   * @return
   *   Newslettter's alternate design.
   */
  public function getAlternateDesign() {
    if (!$this->alt_did) {
      return NULL;
    }
    if (!$this->alternateDesign) {
      $this->alternateDesign = SunMailerDesign::loadDesign($this->alt_did);
    }
    return $this->alternateDesign;
  }

  /**
   * Sets the ID for the newsletter's alternate design.
   *
   * @param $altDid
   *   ID for the newsletter's alternate design.
   */
  public function setAlternateDesignId($altDid) {
    $this->alternateDesign = NULL;
    $this->alt_did = $altDid;
  }

  /**
   * Deletes the newsletter's alternate design.
   */
  public function deleteAlternateDesign() {
    $this->setAlternateDesignId(0);
  }

  /**
   * Gets the newsletter's weight.
   *
   * @return
   *   Newsletter's weight.
   */
  public function getWeight() {
    return $this->weight;
  }

  /**
   * Sets the newsletter's weight.
   *
   * @param $weight
   *   Newsletter's weight.
   */
  public function setWeight($weight) {
    $this->weight = $weight;
  }

  /**
   * Gets the newsletter's sections.
   *
   * @return
   *   Newsletter's sections.
   */
  public function getSections() {
    if ($this->sections === NULL) {
      $this->sections = SunMailerSection::loadNewsletterSections($this->nlid);
    }
    return $this->sections;
  }

  /**
   * Saves the newsletter in the database.
   */
  public function save() {
    // Save the newsletter.
    if ($this->logo_fid) {
      file_set_status($this->getLogo(), FILE_STATUS_PERMANENT);
    }

    $newsletter = new stdClass();
    foreach ($this as $key => $value) {
      $newsletter->$key = $this->$key;
    }
    drupal_write_record('sunmailer_newsletters', $newsletter, $newsletter->nlid ? 'nlid' : array());

    // Delete old files.
    if ($this->oldLogoFid != $this->logo_fid) {
      db_query("DELETE FROM {files} WHERE fid = %d", $this->oldLogoFid);
      $this->oldLogoFid = $this->logo_fid;
    }
  }

  /**
   * Loads a collection of newsletters from the database.
   *
   * @param $query
   *   Query to retrieve the newseletters.
   * @param $args
   *   (optional) Query arguments.
   */
  private static function loadNewsletters($query, $args = array()) {
    $newsletters = array();
    $result = db_query($query, $args);
    while ($row = db_fetch_object($result)) {
      $newsletters[$row->nlid] = new SunMailerNewsletter($row);
    }
    return $newsletters;
  }
}
