<?php
/**
 * @file
 * Contains the SunMailerSubscriber class.
 */

/**
 * Represents a subscriber.
 */
class SunMailerSubscriber implements ISunmailerSubscriber {
  /**
   * SunMailer subscriber ID stored in the database.
   *
   * An entry is only stored for users with a subscription.
   *
   * @var int
   */
  private $smid;

  /**
   * Email address stored in the database.
   *
   * @var string
   */
  private $mail;

  /**
   * IDs of the sections the user subscribes to, grouped newsletter ID.
   *
   * Array keys are only set for newsletters and sections subscribed to.
   *
   * @var array
   */
  private $subscription;

  /**
   * Loads a subscriber from the database.
   *
   * @param $mail
   *   (Optional) Subscriber's email address
   *
   * @return
   *   New or existing subscriber.
   */
  public static function loadSubscriber($mail = '') {
    // First try loading a subscriber from the database.
    $query = "SELECT * FROM {sunmailer_subscribers} WHERE mail = '%s'";
    $subscriber = db_fetch_object(db_query($query, $mail));
    if ($subscriber) {
      // Load the subscription from the database.
      $query = "SELECT se.nlid, se.sid FROM {sunmailer_subscriptions} su, {sunmailer_sections} se WHERE su.sid = se.sid AND smid = %d";
      $result = db_query($query, $subscriber->smid);
      $subscription = array();
      while ($row = db_fetch_object($result)) {
        $subscription[$row->nlid][$row->sid] = $row->sid;
      }
      return new SunMailerSubscriber($subscriber, $subscription);
    }

    // Create a new subscriber.
    $subscriber = new stdClass();
    $subscriber->smid = NULL;
    $subscriber->mail = $mail;
    return new SunMailerSubscriber($subscriber);
  }

  /**
   * Initializes a new instance of the SunMailerSubscriber class.
   *
   * @param unknown_type $subscriber
   *   Object containing the subscriber's data.
   * @param unknown_type $subscription
   *   Object containing the subscriber's subscription.
   */
  private function __construct($subscriber, $subscription = array()) {
    foreach ($subscriber as $key => $value) {
      $this->$key = $subscriber->$key;
    }
    $this->subscription = $subscription;
  }

  /**
   * Gets the subscriber's ID.
   *
   * @return
   *   Subscriber's ID.
   */
  public function getSubscriberId() {
    return $this->smid;
  }

  public function getMail() {
    return $this->mail;
  }

  /**
   * Sets the subscriber's email address.
   *
   * @param $mail
   *   Subscriber's email address.
   */
  public function setMail($mail) {
    $this->mail = $mail;
  }

  /**
   * Gets the subscriber's hash for authentication.
   *
   * @return
   *   Subscriber's hash.
   */
  public function getHash() {
    return md5($this->mail . drupal_get_private_key());
  }

  public function getNewsletterSubscription($nlid) {
    return isset($this->subscription[$nlid]) ? $this->subscription[$nlid] : array();
  }

  /**
   * Sets the subscription for a newsletter.
   *
   * @param $nlid
   *   ID of the newsletter to set the subscription for.
   * @param $sids
   *   IDs of the sections to subscribe to.
   */
  public function setNewsletterSubscription($nlid, $sids) {
    if (!count($sids)) {
      unset($this->subscription[$nlid]);
      return;
    }
    $this->subscription[$nlid] = $sids;
  }

  public function getSubscriptionLink() {
    // Check for an anonymous link.
    if (!$this->getMail()) {
      return url('sunmailer/subscribe', array('absolute' => TRUE));
    }

    // Embed the mail and hash into the link.
    $query = array(
      'mail' => $this->getMail(),
      'hash' => $this->getHash(),
    );
    return url('sunmailer/subscribe', array('absolute' => TRUE, 'query' => $query));
  }

  /**
   * Gets the absolute URL for the page to confirm a subscription change.
   *
   * @param $nlids
   *   IDs of the newsletters to edit the subscription for
   * @param $sids
   *   IDs of the sections to subscribe to for these newsletters
   *
   * @return
   *   Absolute URL for the page to confirm a subscription change.
   */
  public function getConfirmationLink($nlids, $sids) {
    if (!$this->getMail()) {
      return NULL;
    }
    $query = array(
      'mail' => $this->getMail(),
      'hash' => $this->getHash(),
      'nlids' => implode(',', $nlids),
      'sids' => implode(',', $sids),
    );
    return url('sunmailer/subscribe/confirm', array('absolute' => TRUE, 'query' => $query));
  }

  /**
   * Saves the subscriber in the database.
   */
  public function save() {
    // Don't store subscribers without an email address or subscription.
    if (!$this->mail || !count($this->subscription)) {
      if ($this->smid) {
        $query = "DELETE FROM {sunmailer_subscriptions} WHERE smid = %d";
        db_query($query, $this->smid);
        $query = "DELETE FROM {sunmailer_subscribers} WHERE smid = %d";
        db_query($query, $this->smid);
      }
      return;
    }

    // Save the subscriber.
    $subscriber = new stdClass();
    foreach ($this as $key => $value) {
      $subscriber->$key = $this->$key;
    }
    drupal_write_record('sunmailer_subscribers', $subscriber, $subscriber->smid ? 'smid' : array());
    $this->smid = $subscriber->smid;

    // Validate the subscription.
    $placeholders = array();
    $args = array();
    foreach (SunMailerNewsletter::loadAllNewsletters() as $nlid => $newsletter) {
      if (isset($this->subscription[$nlid])) {
        foreach ($newsletter->getSections() as $sid => $section) {
          if (isset($this->subscription[$nlid][$sid])) {
            $placeholders[] = '(%d, %d)';
            $args[] = $this->smid;
            $args[] = $sid;
          }
        }
      }
    }

    // Store the subscription.
    $query = "DELETE FROM {sunmailer_subscriptions} WHERE smid = %d";
    db_query($query, $this->smid);
    if (count($args)) {
      $query = "INSERT INTO {sunmailer_subscriptions}(smid, sid) VALUES " . implode(', ', $placeholders);
      db_query($query, $args);
    }
  }
}